#!/bin/bash
#
# build_lapack.sh 
# (in source build required)
#
# NOTE: The CMake version used for the build is controlled
# below. Look for the version near the package version.
#

usage()
{
   echo "USAGE: `basename ${0}` <product_dir> <e6|e7|e9|e10|e12|e14> <prof> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

die()
{
    local exitval
    if [[ "$1" =~ ^[0-9]*$ ]]; then (( exitval = $1 )); shift; else (( exitval = 1 )); fi
    echo "ERROR: $@" 1>&2
    exit $exitval
}

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
}

set_qualifiers()
{
    if [ -z ${basequal} ]
    then
       echo "ERROR: please specify the qualifiers"
       usage
       exit 1
    fi

    fullqual=${basequal}:${extraqual}
    qualdir=`echo ${basequal} | sed -e 's/:/-/g'`-${extraqual}
    if [ "${extraqual}" = "opt" ]
    then
        die "opt builds of lapack not supported at this time."
	      qualver=`echo ${basequal} | sed -e 's/:/_/g'`_${extraqual}
    elif [ "${extraqual}" = "debug" ]
    then
        die "debug builds of lapack not supported at this time."
	      qualver=${extraqual}_`echo ${basequal} | sed -e 's/:/_/g'`
    elif [ "${extraqual}" = "prof" ]
    then
	      qualver=`echo ${basequal} | sed -e 's/:/_/g'`_${extraqual}
    else
       echo "ERROR: please specify debug, opt, or prof"
       usage
       exit 1
    fi
}

get_platform_name()
{
    OS1=`uname`
    flvr=`ups flavor`
    if [ ${OS1} = "Darwin" ]
    then
      (( darwin = 1 ))
      flvr=`ups flavor -2`
    fi
    OS=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform`
    plat=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform osplat`

    if [ -z ${flvr} ]
    then
       echo "ERROR: could not determine flavor for this platform"
       echo "       something is very wrong"
       exit 1
    fi

    pkgdir=${product_dir}/${package}/${pkgver}/${flvr}-${qualdir}
    patchdir=${product_dir}/${package}/${pkgver}/patch
    blddir=${product_dir}/${package}/${pkgver}/build/${flvr}-${qualdir}
    tardir=${product_dir}/${package}/${pkgver}/tar
    sourcedir=${product_dir}/${package}/${pkgver}/src # Not in tar file because no debug symbols.
}

function setup_and_verify()
{
    # this should not complain
    echo "Finished building ${package} ${pkgver}"
    setup ${package} ${pkgver} -q ${fullqual} -z ${product_dir}:${PRODUCTS}
    echo "${package} is installed at ${LAPACK_FQ_DIR}"
}

function make_osx_prodct_stub()
{
    cd ${pkgdir}/lib64
#create softlinks here
    blas_path=/System/Library/Frameworks/Accelerate.framework/Versions/Current/Frameworks/vecLib.framework/Versions/A/libBLAS.dylib
    lapack_path=/System/Library/Frameworks/Accelerate.framework/Versions/Current/Frameworks/vecLib.framework/Versions/A/libLAPACK.dylib

    ln -s ${blas_path} libblas.dylib
    ln -s ${lapack_path} liblapack.dylib

    ${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

    setup_and_verify

    # this must be last
    if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib64 ]
    then
       ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
    fi
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=lapack
origpkgver=v3_7_0
pkgver=${origpkgver}
ssibuildshims_version=v0_15_09
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' -e 's/^v//'`
pkgtar=lapack-${pkgdotver}.tgz
make_tarball_opts=("\${product_dir}" "\${package}" "\${pkgver}" "\${fullqual}")
cmakevers=v3_5_1

get_this_dir
get_ssibuildshims
set_qualifiers
get_platform_name

if [ "${maketar}" = "tar" ] && ( [ -d ${pkgdir}/lib64 ] ||  [ -d ${pkgdir}/lib ] )
then
   eval ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball "${make_tarball_opts[@]}"
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

mkdir -p ${pkgdir}/lib64 || die "failed to create ${pkgdir}/lib64"

# make a product stub for OSX
if [ "${OS1}" = "Darwin" ]
then
    make_osx_prodct_stub
    exit 0
fi

# Nothing below is run on OSX...

mkdir -p ${sourcedir} || die "failed to create ${blddir}"
mkdir -p ${blddir} || die "failed to create ${blddir}"

# lapack's makefiles can not do an out-of-source build.
cd ${sourcedir} || die "Unable to cd to ${sourcedir}"
echo "In code directory: ${PWD}"
tar xf ${tardir}/${pkgtar} || die "Unable to unwind ${tardir}/${pkgtar} into ${PWD}"

# Apply the patch.
#osname=`uname -s`
#patch -p1 < ${patchdir}/${osname}.patch || die "Failed to apply patch"

# We only make an optimized build, because lapack does not support
# anything else (and nobody would really want it anyway).
cd ${sourcedir}/lapack-${pkgdotver} || die "Unable to cd to ${sourcedir}/lapack-${pkgdotver}"


# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

setup -B ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${product_dir}:${PRODUCTS} || die "fake setup failed"

set -x

cd ${blddir} || die "Unable to cd to ${blddir}"

setup cmake ${cmakevers} || die "Unable to setup cmake"

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

# We would wish to use -std=2008ts, but LAPACK source code doesn't
# support it. Fortunately it doesn't tickle anything that would make it
# binary incompatible with anything we would do downstream.
FC=gfortran \
CC=gcc \
CXX=FALSE \
    cmake \
    -DCMAKE_Fortran_FLAGS="-ffree-line-length-none" \
    -DCMAKE_INSTALL_PREFIX=${LAPACK_FQ_DIR} \
    -DCMAKE_BUILD_TYPE=RELEASE \
    -DBUILD_SHARED_LIBS=ON \
    -DBUILD_STATIC_LIBS=OFF \
    ${sourcedir}/lapack-${pkgdotver} || die "Failed to run CMake"
make -j $ncore || die "Failed in 'make'"
ctest -j $ncore || die "Failed in ctest"
make install || die "Failed to install"
# The Ubuntu build likes to use 'lib' as the subdirectory of
# $LAPACK_FQ_DIR, while SLF likes to use lib64. We'll use lib64.
if [ -f ${LAPACK_FQ_DIR}/lib/liblapack.so ]
then
  mkdir -p ${LAPACK_FQ_DIR}/lib64
  mv ${LAPACK_FQ_DIR}/lib/* ${LAPACK_FQ_DIR}/lib64/
  rm -r ${LAPACK_FQ_DIR}/lib
fi

set +x

if [ ! -f "${LAPACK_LIB}/liblapack.so.${pkgdotver}" ]
then
   echo "ERROR: failed to populate ${LAPACK_LIB}"
   echo "       Something is very wrong"
   exit 1
fi

# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual} || \
  die "failed to declare ${package} ${pkgver} -q ${fullqual}"

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

setup_and_verify

# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${LAPACK_LIB} ]
then
   eval ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball "${make_tarball_opts[@]}"
fi

exit 0
